#include "remstud.h"

#ifdef BIG_ENDIAN

long cnvlong(long x)
{
	unsigned char *s = (unsigned char*)&x;

	return s[0] | (s[1]<<8) | (s[2]<<16) | (s[3]<<24);
}

short cnvshort(short x)
{
	unsigned char *s = (unsigned char*)&x;

	return s[0] | (s[1]<<8);
}

float cnvfloat(float f)
{
	long l = cnvlong(*(long*)&f);
	return *(float*)&l;
}

#else

#define cnvlong(x) (x)
#define cnvshort(x) (x)
#define cnvfloat(x) (x)

#endif

Block::Block()
{
	m_pTime = NULL;
}

void
Block::Read(ifstream &input)
{
	float local_camera[3];
	long t;

	strcpy(m_buffer, "ABCDEFG");
	input.read((char*)&t, 4);
	m_size = cnvlong(t);
	input.read((char*)local_camera, 12);

	camera[0] = cnvfloat(local_camera[0]);
	camera[1] = cnvfloat(local_camera[1]);
	camera[2] = cnvfloat(local_camera[2]);

	input.read(m_buffer, m_size);
	m_buffptr = m_buffer;
}

void
Block::Write(ofstream &output)
{
	float local_camera[3];
	long t = cnvlong(m_size);

	output.write((char*)&t, 4);
	local_camera[0] = cnvfloat(camera[0]);
	local_camera[1] = cnvfloat(camera[1]);
	local_camera[2] = cnvfloat(camera[2]);
	output.write((char*)local_camera, 12);
	output.write(m_buffer, m_size);
}

byte
Block::GetByte()
{
	return *m_buffptr++;
}

long
Block::GetLong()
{
	long temp = *(long*)m_buffptr;
	m_buffptr += 4;
	return cnvlong(temp);
}

float
Block::GetFloat()
{
	long t = GetLong();
	return *(float*)&t;
}

short
Block::GetShort()
{
	short temp = *(short*)m_buffptr;
	m_buffptr += 2;
	return cnvshort(temp);
}

void
Block::GetString()
{
	strcpy(buffer, m_buffptr);
	m_buffptr += strlen(buffer) + 1;
}

void
Block::PutString()
{
	strcpy(m_buffptr, buffer);
	m_buffptr += strlen(buffer) + 1;
	m_size += strlen(buffer) + 1;
}

void
Block::PutByte(byte c)
{
	*(m_buffptr++) = c;
	m_size++;
}

void
Block::PutShort(short a)
{
	*(short*)m_buffptr = cnvshort(a);
	m_buffptr += 2;
	m_size += 2;
}

void
Block::PutLong(long a)
{
	*(long*)m_buffptr = cnvlong(a);
	m_buffptr += 4;
	m_size += 4;
}

void
Block::PutFloat(float a)
{
	PutLong(*(long*)&a);
}

void
Block::Reset()
{
	m_size = 0;
	m_buffptr = m_buffer;
}

float
Block::GetBlockTime()
{
	if (!m_pTime)
	{
		Block Other;
		m_buffptr = m_buffer;
		CamFile().Process(*this, Other); // XXX guessing...
		m_buffptr = m_buffer;
	}
	if (m_pTime) {
		long t = cnvlong(*(long*)m_pTime);
		return *(float*)&t;
	}
	return 0;
}

void
Block::SetBlockTime(float Time)
{
	if (!m_pTime)
	{
		Block Other;
		m_buffptr = m_buffer;
		CamFile().Process(*this, Other); // XXX guessing...
		m_buffptr = m_buffer;
	}
	if (m_pTime) {
		long t = cnvlong(*(long*)&Time);
		*m_pTime = *(float*)&t;
	}
}
